--[[
************************************************************************

tracker.lua

File date: 2009-12-10T17:43:11Z
File revision: 232
Project revision: 233
Project version: r233


Author: Angelike

************************************************************************
]]--

local beql = LibStub("AceAddon-3.0"):GetAddon("beql")
local L = LibStub("AceLocale-3.0"):GetLocale("beql")

local beqlQuestWatchFrame = beqlQuestWatchFrame

function beql:QuestTracker_OnMouseUp(frame, button)
	if button == "LeftButton" and (GetTime() - frame.leftclicktime) < 1 then
		if not beql.db.char.QuestLogMinimizedtracker then
			beql:QuestTrackerClick(frame, button)
		end
	elseif button == "RightButton" and (GetTime() - frame.rightclicktime) < 1 then
		if not beql.db.char.QuestLogMinimizedtracker then
			beql:QuestTrackerClick(frame, button)
		end
	end
end

-- minimize questtracker
function beql:TrackerMinimize()
	beql.db.char.QuestLogMinimizedtracker = true
	beqlQuestWatchFrame.TitleFrame.Minimize:SetNormalTexture("Interface\\AddOns\\beql\\Images\\restore_up")
	beqlQuestWatchFrame.TitleFrame.Minimize:SetPushedTexture("Interface\\AddOns\\beql\\Images\\restore_down")

	-- Hide Lines
	beqlQuestWatchFrame.LineFrame:Hide();

	local minWidth = beqlQuestWatchFrame.TitleFrame.Text:GetStringWidth()+32
	beqlQuestWatchFrame:SetHeight(1)
	beqlQuestWatchFrame:SetWidth(minWidth)
	beqlQuestWatchFrame.Backdrop:Hide()
end

-- maximze questtracker
function beql:TrackerMaximize()
	beql.db.char.QuestLogMinimizedtracker = false
	beqlQuestWatchFrame.TitleFrame.Minimize:SetNormalTexture("Interface\\AddOns\\beql\\Images\\minimize_up")
	beqlQuestWatchFrame.TitleFrame.Minimize:SetPushedTexture("Interface\\AddOns\\beql\\Images\\minimize_down")

	beqlQuestWatchFrame.LineFrame:Show()
	beqlQuestWatchFrame.Backdrop:Show()
	WatchFrame_Update()
end

-- Toggles Questtracker status between active and inactive
function beql:ToggleQuestTracker(inactive)
	beql.db.profile.disabledtracker = inactive

	-- unhook old scripts
	local hooks = {'AddQuestWatch','RemoveQuestWatch','IsQuestWatched','GetNumQuestWatches','GetQuestIndexForWatch',}
	for k in ipairs(hooks) do
		beql:Unhook(hooks[k]);
	end

	-- unregister Callbacks
	local callbacks = {'Quest_Gained','Update','Ready','Quest_Lost',}
	for k in ipairs(callbacks) do
		beqlQ.UnregisterCallback(beql, callbacks[k]);
	end

	if(not inactive) then
		-- load tracker
		beql:debugprint("load questtracker")
		if not (beqlQuestWatchFrame) then
			beql:InitTracker()
		end

		-- Callbacks
		for k in ipairs(callbacks) do
			beqlQ.RegisterCallback(beql, callbacks[k], beql['Quixote_' .. callbacks[k]]);
		end

		WatchFrame_RemoveObjectiveHandler(WatchFrame_DisplayTrackedQuests);
		WatchFrame_AddObjectiveHandler(beql.DisplayTrackedQuests);

		-- make some hooks
		for k in ipairs(hooks) do
			beql:RawHook(hooks[k], 'Hooks_' .. hooks[k], true);
		end

		beqlQuestWatchFrame:Show()

		WATCHFRAME_MAXQUESTS = 25;
		MAX_WATCHABLE_QUESTS = 25;

		beql:debugprint("questracker loaded")
	elseif( inactive ) then

		-- unload tracker
		WatchFrame_RemoveObjectiveHandler(beql.DisplayTrackedQuests);
		WatchFrame_AddObjectiveHandler(WatchFrame_DisplayTrackedQuests);


		--beqlQuestWatchFrame:Hide();

		WATCHFRAME_MAXQUESTS = 10;
		MAX_WATCHABLE_QUESTS = 10;

		if not WatchFrame:IsVisible() then
			WatchFrame:Show()
		end
	end

	QuestLog_Update();
	WatchFrame_ClearDisplay();
	WatchFrame_Update(WatchFrame);

end

local nametable = {}
local playerlevel, showzones

function beql:SortWatchedQuests()
	local GetQuestLogTitle = beql.hooks.GetQuestLogTitle or GetQuestLogTitle
	local NumQuestLogEntries = GetNumQuestLogEntries()
	if NumQuestLogEntries == 0 then
		return
	end

	if playerlevel ~= UnitLevel("player") or beql.db.profile.QuestTracker.ShowZones ~= showzones then
		showzones = beql.db.profile.QuestTracker.ShowZones
		playerlevel = UnitLevel("player")
		table.wipe(nametable)
	end

	-- cleanup unused uid's
	local changed, rebuild
	while ( not changed ) do
		changed = 1
		for i=1, #beql.db.char.QuestWatchIDs do
			-- rebuild table if key not exist
			if not nametable[beql.db.char.QuestWatchIDs[i]] then
				rebuild = true
			end
			if not beql.uID_2_index[beql.db.char.QuestWatchIDs[i]] then
				tremove(beql.db.char.QuestWatchIDs,i)
				changed = nil
				break
			end
		end
	end

	if rebuild then
		local curheader, uID
		for i=1, NumQuestLogEntries do
			uID = select(9,GetQuestLogTitle(i))
			if uID == 0 then
				curheader = beql.db.profile.QuestTracker.ShowZones and (GetQuestLogTitle(i) .. ".") or "";
			else
				for k in ipairs(beql.db.char.QuestWatchIDs) do
					if beql.db.char.QuestWatchIDs[k] == uID and not nametable[uID] then
						local name, level = GetQuestLogTitle(i);
						tinsert(nametable, uID, curheader .. level .. "." .. name);
					end
				end
			end
		end
	end

	table.sort(beql.db.char.QuestWatchIDs, function(a,b)
		if nametable[a] < nametable[b] then
			return true
		end
		return false
	end)
end

-- Update Base Stats of the Frame like Color etc...
local FontHeight, FontSpacing, QuestSpacing, ZoneSpacing
function beql:UpdateQuestTrackerFrame()
	if not beql.db then
		-- DB not loaded, exit
		return nil
	end
	
	FontHeight, FontSpacing, QuestSpacing, ZoneSpacing = nil, nil, nil, nil
	local QuestTrackerOptions = beql.db.profile.QuestTracker

	beql:debugprint("beql:UpdateQuestTrackerFrame")

	-- movability
	if QuestTrackerOptions.locked then
		beqlQuestWatchFrame:RegisterForDrag(0)
		beqlQuestWatchAnchor:SetMovable(nil);
	else
		beqlQuestWatchFrame:RegisterForDrag("LeftButton")
		beqlQuestWatchAnchor:SetMovable(true);
	end

	-- Title
	local title = beqlQuestWatchFrame.TitleFrame
	if QuestTrackerOptions.ShowHeader then
		title:Show();
	else
		title:Hide();
	end

	if QuestTrackerOptions.locked then
		beqlQuestWatchFrameTitle:RegisterForDrag(0);
		title:EnableMouse(nil);
	else
		beqlQuestWatchFrameTitle:RegisterForDrag("LeftButton");
		title:EnableMouse(true);
	end

	if QuestTrackerOptions.UseBgColors then
		if QuestTrackerOptions.CustomBgColor then
			title:SetBackdropColor(
				QuestTrackerOptions.Colors.Bg.r,
				QuestTrackerOptions.Colors.Bg.g,
				QuestTrackerOptions.Colors.Bg.b,
				QuestTrackerOptions.Colors.Bg.a
			)
			title:SetBackdropBorderColor(
				QuestTrackerOptions.Colors.BgBorder.r,
				QuestTrackerOptions.Colors.BgBorder.g,
				QuestTrackerOptions.Colors.BgBorder.b,
				QuestTrackerOptions.Colors.BgBorder.a
			)
		else
			title:SetBackdropBorderColor(
				TOOLTIP_DEFAULT_BACKGROUND_COLOR.r,
				TOOLTIP_DEFAULT_BACKGROUND_COLOR.g,
				TOOLTIP_DEFAULT_BACKGROUND_COLOR.b,
				1
			)
			title:SetBackdropColor(
				TOOLTIP_DEFAULT_BACKGROUND_COLOR.r,
				TOOLTIP_DEFAULT_BACKGROUND_COLOR.g,
				TOOLTIP_DEFAULT_BACKGROUND_COLOR.b,
				1
			)
		end
	else
		title:SetBackdropColor(0,0,0,0)
		title:SetBackdropBorderColor(0,0,0,0)
	end

	local titletext = title.Text
	local t1, _, t2 = titletext:GetFont();
	titletext:SetFont(t1, QuestTrackerOptions.FontHeight+2, t2)
	titletext:SetHeight(QuestTrackerOptions.FontHeight+2)


	-- Frame
	if QuestTrackerOptions.UseBgColors then
		if QuestTrackerOptions.CustomBgColor then
			beqlQuestWatchFrame.Backdrop:SetBackdropColor(
				QuestTrackerOptions.Colors.Bg.r,
				QuestTrackerOptions.Colors.Bg.g,
				QuestTrackerOptions.Colors.Bg.b,
				QuestTrackerOptions.Colors.Bg.a
			)
			beqlQuestWatchFrame.Backdrop:SetBackdropBorderColor(
				QuestTrackerOptions.Colors.BgBorder.r,
				QuestTrackerOptions.Colors.BgBorder.g,
				QuestTrackerOptions.Colors.BgBorder.b,
				QuestTrackerOptions.Colors.BgBorder.a
			)
		else
			beqlQuestWatchFrame.Backdrop:SetBackdropBorderColor(
				TOOLTIP_DEFAULT_BACKGROUND_COLOR.r,
				TOOLTIP_DEFAULT_BACKGROUND_COLOR.g,
				TOOLTIP_DEFAULT_BACKGROUND_COLOR.b,
				1
			)
			beqlQuestWatchFrame.Backdrop:SetBackdropColor(
				TOOLTIP_DEFAULT_BACKGROUND_COLOR.r,
				TOOLTIP_DEFAULT_BACKGROUND_COLOR.g,
				TOOLTIP_DEFAULT_BACKGROUND_COLOR.b,
				1
			)
		end
	else
		beqlQuestWatchFrame.Backdrop:SetBackdropColor(0,0,0,0)
		beqlQuestWatchFrame.Backdrop:SetBackdropBorderColor(0,0,0,0)
	end

	-- Alpha
	beqlQuestWatchFrame:SetAlpha(beql.db.profile.QuestTracker.Alpha)
	
	if beqlQuestWatchFrame:IsVisible() then
		WatchFrame_Update();
	end

	return 1;
end

--
-- Events
--

function beql:Quixote_Ready()
	WatchFrame_Update()
end

function beql:Quixote_Update()
	beql:SortWatchedQuests()
	WatchFrame_Update()
end

function beql:tracker_Quest_Gained( questname, qID, objectives, category, provider, player)
	local _ , id, _ ,_ , _ ,_ , complete ,_ , _ ,_ = beqlQ:GetQuestByUid(qID)

	if(complete == 1) and qID == beql.db.char.newquestid then
		beql:Quixote_Quest_Complete(questname, uid)
	end

	if beql.db.profile.QuestTracker.AddNew and qID == beql.db.char.newquestid then
		AddQuestWatch(id)
		QuestLog_Update()
		WatchFrame_Update()
	end
	beql.WatchQueue = nil
end

--
-- Hooks
--

function beql:Hooks_AddQuestWatch(questIndex)
	local uID
	if questIndex and type(questIndex) == 'string' and strfind(questIndex, 'uID:') then
		uID = tonumber(string.match(questIndex, '%d+'))
		if not uID or not beql.uID_2_index[uID] then
			assert(false, "usage: AddQuestWatch(index or 'uID:uID') - invalid uID");
			return;
		end
	elseif questIndex and type(questIndex) == 'number' then
		uID = select(9, GetQuestLogTitle(questIndex))
		if uID == 0 then
			assert(false, "usage: AddQuestWatch(index or 'uID:uID') - invalid index");
			return;
		end
	else
		assert(false, "usage: AddQuestWatch(index or 'uID:uID')")
		return;
	end

	for k in pairs(beql.db.char.QuestWatchIDs) do
		if beql.db.char.QuestWatchIDs[k] == uID then
			return;
		end
	end
	tinsert(beql.db.char.QuestWatchIDs, uID)
end

function beql:Hooks_RemoveQuestWatch(questIndex)
	local uID
	if questIndex and type(questIndex) == 'string' and strfind(questIndex, 'uID:') then
		uID = tonumber(string.match(questIndex, '%d+'))
		if not uID or not beql.uID_2_index[uID] then
			assert(false, "usage: RemoveQuestWatch(index or 'uID:uID') - invalid uID");
			return;
		end
	elseif questIndex and type(questIndex) == 'number' then
		uID = select(9, GetQuestLogTitle(questIndex))
		if uID == 0 then
			assert(false, "usage: RemoveQuestWatch(index or 'uID:uID') - invalid index");
			return;
		end
	else
		assert(false, "usage: RemoveQuestWatch(index or 'uID:uID')")
		return;
	end

	local numQuestWatches = #beql.db.char.QuestWatchIDs
	for i=1, numQuestWatches do
		if beql.db.char.QuestWatchIDs[i] == uID then
			tremove(beql.db.char.QuestWatchIDs, i)
			return
		end
	end
end

function beql:Hooks_IsQuestWatched(questIndex)
	local uID
	if questIndex and type(questIndex) == 'string' and strfind(questIndex, 'uID:') then
		uID = tonumber(string.match(questIndex, '%d+'))
		if not uID or not beql.uID_2_index[uID] then
			assert(false, "usage: IsQuestWatched(index or 'uID:uID') - invalid uID");
			return;
		end
	elseif questIndex and type(questIndex) == 'number' then
		uID = select(9, GetQuestLogTitle(questIndex))
		local questname = GetQuestLogTitle(questIndex)
		if uID == 0 and questname == nil then
			assert(false, "usage: IsQuestWatched(index or 'uID:uID') - invalid index");
			return;
		end
	else
		assert(false, "usage: IsQuestWatched(index or 'uID:uID')")
		return;
	end

	local numQuestWatches = #beql.db.char.QuestWatchIDs
	for i=1, numQuestWatches do
		if beql.db.char.QuestWatchIDs[i] == uID then
			return true;
		end
	end
	return false;
end

function beql:Hooks_GetNumQuestWatches()
	return table.getn(beql.db.char.QuestWatchIDs)
end

function beql:Hooks_GetQuestIndexForWatch(watchindex)
	if not watchindex or type(watchindex) ~= 'number' then
		assert(false, "usage: GetQuestIndexForWatch(index)");
		return 0;
	end

	if watchindex > table.getn(beql.db.char.QuestWatchIDs) then
		assert(false, "usage: GetQuestIndexForWatch(index) - invalid index");
		return 0;
	end

	local uID, questIndex

	uID = beql.db.char.QuestWatchIDs[watchindex]
	questIndex = beql.uID_2_index[uID] or 0

	return questIndex, uID
end

function beql:Hooks_WatchFrame_Update()

	-- Hide empty WatchFrame
	if 	-- Timer
		select('#',GetQuestTimers()) == 0
	and -- Achievements
		((select('#', GetTrackedAchievements()) == 0 and not beql.db.profile.AchievementTracker.Enable) or beql.db.profile.AchievementTracker.Enable)
	and -- Quests
		((GetNumQuestWatches() == 0 and beql.db.profile.disabledtracker) or not beql.db.profile.disabledtracker)
	then
		WatchFrame:Hide()
	else
		WatchFrame:Show()
	end
end

local WATCHFRAMELINES_FONTSPACING = 0

function beql:DisplayTrackedQuests ()
	-- Don't do anything before Quixote gets his data
	if not beqlQ.firstDone then
		return 0,0
	end

	-- HACK 3.3
	local WATCHFRAME_QUEST_WITH_ITEM_HEIGHT = 48;
	
	-- beql locales
	local questIndex,isRemoved,isCollapsed
	local qTitle,qLvl,qTag,qRec,qStat,qObj,qZone,qID, qdaily
	local oText, oType, oNumP, oNumN, oComp, numObjectives, objText
	local temp,curZone
	local zoneid = 0
	local Color = {}
	local tempwidth = 0
	local tempColor,tempColor2,tempColor3
	local markerID = 0
	local completedQuests = 0
	local questWatchMaxWidth = 0
	local _
	local qObjectives, oItems

--	local zoneoffset = 5
--	local questoffset = 3.5

	beql.db.char.TrackerQuests = {}

	-- blizz locales
	local lineFrame = beqlQuestWatchFrame.LineFrame
	local _;
	local text, finished;
	local questTitle;
	local watchItemIndex = 0;

	local line;
	local lastLine;
	local HIGHLIGHT_FONT_COLOR = HIGHLIGHT_FONT_COLOR;
	local NORMAL_FONT_COLOR = NORMAL_FONT_COLOR;
	local WATCHFRAMELINES_FONTHEIGHT = beql.db.profile.QuestTracker.FontHeight

	local maxWidth = 0;
	local questwidth = 0

	local iconHeightLeft = 0;
	local questLogTitleText

	local maxtrackerwidth, maxheaderwidth, maxiconwidth

	if not beql.db.profile.QuestTracker.AutoResize then
		maxtrackerwidth = beql.db.profile.QuestTracker.FixedWidth
		if ( SHOW_QUEST_OBJECTIVES_ON_MAP == "1" ) then
			maxheaderwidth = beql.db.profile.QuestTracker.FixedWidth - WATCHFRAME_MAP_BUTTON_WIDTH - 8
		else
			maxheaderwidth = beql.db.profile.QuestTracker.FixedWidth
		end


	end

	for i = 1, GetNumQuestWatches() do
		questWidth = 0;
		questIndex = GetQuestIndexForWatch(i);

		qID = nil;
		if questIndex then
			questLogTitleText = GetQuestLogTitle(questIndex, 'AddTags')
			-- Fix submitted by Ben Schreiber for MarsQuestOrganizer
			if questLogTitleText then
				qID, _ , qTitle, qLvl, qTag, qObj, qStat, qRec, qdaily, qZone = beqlQ:GetQuestById(questIndex)
				-- _ is questIndex in Questlog
			end
		end

		if qID then
			isRemoved = false

			if beql.db.profile.QuestTracker.RemoveFinished and qStat == 1 then
				isRemoved = true
				completedQuests = completedQuests +1
			end

			--
			-- Zone !
			--
			if beql.db.profile.QuestTracker.ShowZones and (curZone == nil or curZone ~= qZone) and not isRemoved then
				curZone = qZone

				line = beql:GetQuestLine(lineFrame:GetName());
				line.text:SetText(curZone);
				if ((line.text:GetStringWidth() + 10) > questWatchMaxWidth) then
					questWatchMaxWidth = line.text:GetStringWidth() +10
				end
				line:Show();
				if ( not lastLine ) then -- First line
					line:SetPoint("TOPRIGHT", lineFrame, "TOPRIGHT", 0, 0);
					line:SetPoint("TOPLEFT", lineFrame, "TOPLEFT", 0, 0);
				else
					local yOffset = 0;
					if ( iconHeightLeft > 0 ) then
						yOffset = -iconHeightLeft;
						iconHeightLeft = 0;
					end

					line:SetPoint("TOPRIGHT", lastLine, "BOTTOMRIGHT", 0, yOffset - ZoneSpacing);
					line:SetPoint("TOPLEFT", lastLine, "BOTTOMLEFT", 0, yOffset - ZoneSpacing);
				end

				if beql.db.profile.QuestTracker.CustomZoneColor then
					Color = beql.db.profile.QuestTracker.Colors.Zone
				else
					Color = {r = 0.8,g = 0.8,b = 1}
				end

				line.text:SetTextColor(Color.r, Color.g, Color.b)
				if not(tempwidth == nil) and tempwidth > questWatchMaxWidth then
					questWatchMaxWidth = tempwidth
				end

				maxWidth = max(maxWidth, line.text:GetStringWidth())
				lastLine = line;
			end

			--
			-- Quest !
			--
			if qID > 0 and not isRemoved then
				-- Colores
				if qStat == 1 then
					--Completed
					completedQuests = completedQuests +1
					if beql.db.profile.QuestTracker.CustomHeaderColor then
						if beql.db.profile.QuestTracker.HeaderQuestLevelColor then
							Color = GetQuestDifficultyColor(qLvl)
						else
							Color = beql.db.profile.QuestTracker.Colors.HeaderComplete
						end
					else
						Color = NORMAL_FONT_COLOR
					end
				elseif qStat == -1 then
					-- Failed
					if beql.db.profile.QuestTracker.CustomHeaderColor then
						Color = beql.db.profile.QuestTracker.Colors.HeaderFailed
					else
						Color = NORMAL_FONT_COLOR
					end
				else
					-- Normal
					if beql.db.profile.QuestTracker.CustomHeaderColor then
						if beql.db.profile.QuestTracker.HeaderQuestLevelColor then
							Color = GetQuestDifficultyColor(qLvl)
						else
							Color = beql.db.profile.QuestTracker.Colors.HeaderNormal
						end
					else
						Color = { r = 0.75, g = 0.61, b = 0 }
					end
				end

				qTitle = questLogTitleText;-- Levels

				if beql.db.profile.QuestTracker.ShowZones then
					-- add space for zone offset
					qTitle = "  " .. qTitle
				end

				line = beql:GetQuestLine(lineFrame:GetName());
				questTitle = line;
				line.text:SetText(qTitle);
				line.qID = qID;
				line.text:SetTextColor(Color.r, Color.g, Color.b)
				line:Show();
				if ( not lastLine ) then -- First line
					line:SetPoint("TOPRIGHT", lineFrame, "TOPRIGHT", 0, 0);
					line:SetPoint("TOPLEFT", lineFrame, "TOPLEFT", 0, 0);
				else
					local yOffset = 0;
					if ( iconHeightLeft > 0 ) then
						yOffset = -iconHeightLeft;
					end

					line:SetPoint("TOPRIGHT", lastLine, "BOTTOMRIGHT", 0, yOffset - QuestSpacing);
					line:SetPoint("TOPLEFT", lastLine, "BOTTOMLEFT", 0, yOffset - QuestSpacing);
				end
				iconHeightLeft = 0;
				local stringWidth = line.text:GetStringWidth();

				--
				-- ItemButton
				--
				local item, charges = select(2,GetQuestLogSpecialItemInfo(questIndex));
				local itemButton;
				if ( item ) then
					watchItemIndex = watchItemIndex + 1;
					itemButton = _G["beqlItem"..watchItemIndex];
					if ( not itemButton ) then
						beql.QuestItems = watchItemIndex
						itemButton = CreateFrame("BUTTON", "beqlItem" .. watchItemIndex, lineFrame, "WatchFrameItemButtonTemplate");
					end
					itemButton:Show();
					itemButton:ClearAllPoints();
					itemButton:SetID(questIndex);
					SetItemButtonTexture(itemButton, item);
					SetItemButtonCount(itemButton, charges);
					itemButton.charges = charges;
					WatchFrameItem_UpdateCooldown(itemButton);
					itemButton.rangeTimer = -1;
					if ( SHOW_QUEST_OBJECTIVES_ON_MAP ~= "1" ) then
						line.text:SetPoint("RIGHT", itemButton, "LEFT", -4, 0);
					end
					iconHeightLeft = WATCHFRAME_QUEST_WITH_ITEM_HEIGHT - WATCHFRAMELINES_FONTHEIGHT - WATCHFRAMELINES_FONTSPACING; -- We've already displayed a line for this
					if ( SHOW_QUEST_OBJECTIVES_ON_MAP == "1" ) then
						itemButton.maxStringWidth = stringWidth + WATCHFRAME_MAP_BUTTON_WIDTH;
					else
						itemButton.maxStringWidth = stringWidth
					end
				end

				lastLine = line;

				--
				-- Map Button (not active now, but maybe in future)
				--
				if ( SHOW_QUEST_OBJECTIVES_ON_MAP == "1" ) then
					stringWidth = stringWidth + WATCHFRAME_MAP_BUTTON_WIDTH;
					line.mapButton:Show();
					line.mapButton.questID = qID;
					line.text:ClearAllPoints();
					line.text:SetPoint("TOPLEFT");
					line.text:SetPoint("BOTTOM");
					local lineWidth = line:GetWidth();
					if maxtrackerwidth and ( line.text:GetWidth() + 4 + WATCHFRAME_MAP_BUTTON_WIDTH > maxtrackerwidth ) then -- 25 = button (22) + Buttonofset (3)
						line.text:SetWidth(maxtrackerwidth - (4 + WATCHFRAME_MAP_BUTTON_WIDTH));
					end
				end
				questWidth = max(stringWidth, questWidth);

				--
				-- Objectives
				--

				numObjectives = qObj
				if (beql.db.profile.QuestTracker.MinimizeFinished or beql.db.profile.QuestTracker.HideCompletedOnly) and qStat == 1 then
					-- hide questobjectives on complete quest
					numObjectives = 0;
				end

				markerID = 0


				local _, qObjectives = beqlQ:IterateObjectivesForQuest(qID)
				if  numObjectives > 0 and qObjectives then
					for oText, oItems in pairs(qObjectives) do
						oComp = false;
						oNumP, oNumN, oType = unpack(oItems)
						if oNumN == nil then oNumN = 1 end
						if oNumP == nil then oNumP = 1 end
						if oText == nil then oText = "" end

						if oNumP == oNumN then
							oComp = true
						end

						-- only display complete
						if (oComp and not beql.db.profile.QuestTracker.MinimizeFinished) or (not oComp) then

							-- colores
							if beql.db.profile.QuestTracker.CustomObjetiveColor then
										tempColor = { r=beql.db.profile.QuestTracker.Colors.ObjectiveNormal.r, g=beql.db.profile.QuestTracker.Colors.ObjectiveNormal.g, b=beql.db.profile.QuestTracker.Colors.ObjectiveNormal.b }
										tempColor2 = { r=beql.db.profile.QuestTracker.Colors.ObjectiveComplete.r, g=beql.db.profile.QuestTracker.Colors.ObjectiveComplete.g, b=beql.db.profile.QuestTracker.Colors.ObjectiveComplete.b }
							else
										tempColor = {r=0.8, g=0.8, b=0.8}
										tempColor2 = {r=HIGHLIGHT_FONT_COLOR.r, g=HIGHLIGHT_FONT_COLOR.g, b=HIGHLIGHT_FONT_COLOR.b}
							end
							if oComp then
								Color = tempColor2
							else
								if beql.db.profile.QuestTracker.FadeColor then
									if oType == "reputation" then
										if oNumN == oNumP then
											Color = beql:FadeColors(tempColor, tempColor2, 1,1)
										else
											Color = beql:FadeColors(tempColor, tempColor2, 0,1)
										end
									else
										Color = beql:FadeColors(tempColor, tempColor2, oNumP,oNumN)
									end
								else
									Color = tempColor
								end
							end

							-- markers
							if  beql.db.profile.QuestTracker.ShowObjectiveMarkers  then
								if beql.db.profile.QuestTracker.UseTrackerListing then
									objText = "   "..BEQL_TrackerLists[beql.db.profile.QuestTracker.TrackerList][markerID]..") "

								else
									objText = "   "..BEQL_TrackerSymbols[beql.db.profile.QuestTracker.TrackerSymbol].." "
								end
							else
								if IsAddOnLoaded("Mobmap") then
									objText = " - "
								else
									objText = "   "
								end
							end

							if beql.db.profile.QuestTracker.ShowZones then
								-- add space for zone offset
								objText = "  " .. objText
							end

							objText = objText..oText
							if oType == "item" or oType == "monster" or oType == "reputation" or oType == "object"then
								objText = objText ..": "..oNumP.."/"..oNumN
							end

							-- create line
							line = beql:GetQuestLine(lineFrame:GetName());
							line.text:SetText(objText);
							line.qID = qID;
							line.text:SetTextColor(Color.r, Color.g, Color.b)
							line:SetPoint("TOPRIGHT", lastLine, "BOTTOMRIGHT", 0, WATCHFRAMELINES_FONTSPACING);
							line:SetPoint("TOPLEFT", lastLine, "BOTTOMLEFT", 0, WATCHFRAMELINES_FONTSPACING);
							line:Show();
							stringWidth = line.text:GetStringWidth();
							questWidth = max(stringWidth, questWidth);
							if ( iconHeightLeft > 0 ) then
								line.text:SetPoint("RIGHT", itemButton, "LEFT", -4, 0);
								itemButton.maxStringWidth = max(stringWidth, itemButton.maxStringWidth)
							end
							lastLine = line;
							iconHeightLeft = iconHeightLeft - WATCHFRAMELINES_FONTHEIGHT - WATCHFRAMELINES_FONTSPACING;
--							if tempwidth > questWatchMaxWidth then
--								questWatchMaxWidth = tempwidth
--							end
							--beql.db.char.TrackerQuests[(beqlFrameCache.QuestLineIndex - 1)] = qID
							markerID = markerID + 1
						end

					end
				end


				-- adjust item button
				local itemButtonXOffset = 8;
--				if ( SHOW_QUEST_OBJECTIVES_ON_MAP == "1" ) then
--					questWidth = questWidth-- + WATCHFRAME_MAP_BUTTON_WIDTH;
					--itemButtonXOffset = itemButtonXOffset + WATCHFRAME_MAP_BUTTON_WIDTH;
--				end

				if ( itemButton ) then
					questWidth = questWidth + WATCHFRAME_ITEM_WIDTH + itemButtonXOffset;
					if maxtrackerwidth then
						-- we have a absolute tracker width
						itemButtonXOffset = min(itemButton.maxStringWidth + itemButtonXOffset, maxtrackerwidth - WATCHFRAME_ITEM_WIDTH);
					else
						itemButtonXOffset = itemButton.maxStringWidth + itemButtonXOffset;
					end
					itemButton:SetPoint("TOPLEFT", questTitle, "TOPLEFT", itemButtonXOffset, -WATCHFRAMELINES_FONTSPACING);
					itemButton:Show();
					if ( SHOW_QUEST_OBJECTIVES_ON_MAP == "1" and (questTitle.text:GetWidth() + WATCHFRAME_MAP_BUTTON_WIDTH) > itemButtonXOffset ) then
						questTitle.text:SetPoint("RIGHT", itemButton, "LEFT", -WATCHFRAME_MAP_BUTTON_WIDTH, 0);
					end
				end
				maxWidth = max(questWidth, maxWidth);

			end
		end

	end

	-- hide unused buttons
	for i = watchItemIndex + 1, beql.QuestItems do
		_G["beqlItem" .. i]:Hide();
	end

	-- Handle Frame Visibility
	local lastline
	if beqlFrameCache.QuestLineIndex > 1 then
		lastline = beqlFrameCache.QuestLineIndex - 1
		beqlQuestWatchFrame:Show()
	else
		beqlQuestWatchFrame:Hide()
	end

	beql:ReleaseUnusedQuestLines();

	if not lastline then
		-- frame is empty and hidden, exit
		return 0,0
	end

	-- Set Title Text
	local questnums = GetNumQuestWatches()
	local _,questEntries = GetNumQuestLogEntries()
	if beql.db.profile.QuestTracker.RemoveFinished then
		beqlQuestWatchFrame.TitleFrame.Text:SetText(L["Quest Tracker"].. " ("..questnums - completedQuests.."/"..questEntries..")")
	else
		beqlQuestWatchFrame.TitleFrame.Text:SetText(L["Quest Tracker"].. " ("..questnums.."/"..questEntries..")")
	end

	--Minimize the Tracker if its state is set to minimized
	if beql.db.char.QuestLogMinimizedtracker then
		beql:TrackerMinimize()
		return 0,0
	end

	-- set Tracker Width
	if not beql.db.profile.QuestTracker.AutoResize then
		beqlQuestWatchFrame:SetWidth(beql.db.profile.QuestTracker.FixedWidth)
	else
		if maxWidth > (beqlQuestWatchFrame.TitleFrame.Text:GetStringWidth()+32) then
			beqlQuestWatchFrame:SetWidth(maxWidth);
		else
			beqlQuestWatchFrame:SetWidth(beqlQuestWatchFrame.TitleFrame.Text:GetStringWidth()+32);
		end
	end

	-- Tracker height
	beqlQuestWatchFrame:SetHeight((beqlFrameCache.QuestLines[1]:GetTop() - beqlFrameCache.QuestLines[lastline]:GetBottom()) + 5)

	-- last item is an icon, expand size to it
	if ( iconHeightLeft > 0 ) then
		beqlQuestWatchFrame:SetHeight(beqlQuestWatchFrame:GetHeight() + iconHeightLeft + 2)
		--yOffset = -iconHeightLeft;
	end


	-- we use a own Frame, so we return a used hight of zero
	return 0, maxWidth;
end

function beql:GetQuestLine (frameparent)
	if not FontHeight or not FontSpacing or not QuestSpacing or not ZoneSpacing then
		FontHeight = beql.db.profile.QuestTracker.FontHeight
		FontSpacing = FontHeight / 6
		QuestSpacing = (FontHeight + FontSpacing) / 4
		ZoneSpacing = FontHeight/2.4
	end




	local line = beqlFrameCache.QuestLines[beqlFrameCache.QuestLineIndex];
	if ( not line ) then
		beqlFrameCache.QuestLines[beqlFrameCache.QuestLineIndex] = beqlFrameCache.Lines:GetFrame();
		line = beqlFrameCache.QuestLines[beqlFrameCache.QuestLineIndex];
	end

	line:Reset();
	line:SetParent(frameparent);

	local t1,_ , t2 = line.text:GetFont()
	line.text:SetFont(t1, FontHeight, t1)
	line:SetHeight(FontHeight + FontSpacing)

	beqlFrameCache.QuestLineIndex = beqlFrameCache.QuestLineIndex + 1;
	return line;
end

function beql:ReleaseUnusedQuestLines ()
	local line
	for i = beqlFrameCache.QuestLineIndex, #beqlFrameCache.QuestLines do
		line = beqlFrameCache.QuestLines[i];
		line:Hide();
		line.frameCache:ReleaseFrame(line);
		beqlFrameCache.QuestLines[i] = nil;
	end

	beqlFrameCache.QuestLineIndex = 1;
end

-- EOF --